/* $Id: glaceG.c,v 1.3 2000/06/08 09:44:15 acapnotic Exp $ */
/* :PREAMBLE: GlaceG.c
 *
 * Wrapper-specific Code:
 *           GIMP plug-in version
 *
 * Based on glaceT.c and glaceP.c by J.Alex Stark
 * g_ified by Kevin M. Turner
 */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.  See the file COPYING for details.
 *
 */

/* :HEADERS:
 */
#include <config.h>

#include <gtk/gtk.h> /* for gtk_preview manipulations */
#include <ctype.h>   /* for tolower in keymatch */
#include <string.h>
#include "glaceInt.h"
#include "preview.h"


/* :*** Internal Functions ***:
 * GlaceWMalloc,Realloc,Calloc, and Free
 * have been implemented as defines in an #ifdef block in glaceInt.h
 * Ugly, but true.
 */

/**************************************************************/

/* :*** Errors and Messages ***:

 * :FUNCTION: Glace_WError
 */
int		      /* Sets error and returns error flag (but may exit) */
Glace_WError (
	       Glace_WData wData,
	       char *argErrStr
)
{
  if (wData == NULL)
    g_warning ("Fatal error (error rountine called"
	     " with invalid package data structure.");
  WRAPPER (errorValue) = GLACE_ERROR;

  g_warning (argErrStr);

  return GLACE_ERROR;
}

/*
 * :FUNCTION: Glace_WErrorValue
 */
int				/* Returns error flag */
Glace_WErrorValue (
		    Glace_WData wData
)
{
  return WRAPPER (errorValue);
}



/*
 * :FUNCTION: Glace_WIsError
 */
int				/* Returns boolean */
Glace_WIsError (
		 Glace_WData wData
)
{
  return (WRAPPER (errorValue) == GLACE_ERROR);
}



/*
 * :FUNCTION: Glace_WMessage
 */
void
Glace_WMessage ( Glace_WData wData G_GNUC_UNUSED,
		 char *messageStr)
{
  g_print("%s", messageStr);
}


/**************************************************************/
/* :*** Wrapper-specific Setup ***:

 * :FUNCTION: Glace_WInit
 */
void
Glace_WInit (Glace_WData wData G_GNUC_UNUSED)
{
#ifdef __EMX__
  _fsetmode (stdin, "b");
  _fsetmode (stdout, "b");
#endif
}



/**************************************************************/
/*
 * :FUNCTION: Glace_WWrapTell
 */
Glace_WrapTypes
Glace_WWrapTell()
{
  return GLACE_GIMP_VER;
}

/**************************************************************/
/*
 * :FUNCTION: Glace_WDataAlloc
 */
Glace_WData
Glace_WDataAlloc ()
{
  return g_new0(GlaceGimpData,1);
}

/**************************************************************/
/*
 * :FUNCTION: Glace_WUsage
 */
void
Glace_WUsage(
	     Glace_WData wData,
    char* usage
)
{
  Glace_WMessage(wData, usage);
  WRAPPER(errorValue) = GLACE_ERROR;
}

/* :*** Image Input and Output ***:

 * :FUNCTION: Glace_WOpenImage 
 */
void
Glace_WOpenImage(Glace_WData wData G_GNUC_UNUSED,
		 Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		 char *name G_GNUC_UNUSED,
		 Glace_ImageHandle *handlePtr G_GNUC_UNUSED )
{
  if(name==NULL) return;
  g_warning("Glace_WOpenImage(%s): function not implemented in glaceG.c",
	    name);
}

/**************************************************************/

/*
 * :FUNCTION: FIXME Glace_WPutImgStart
 * Here we initalize a pixel region to write to, 
 * and set the output pointer to the temp space.
 */
void
Glace_WPutImgStart(Glace_WData wData, Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		   Glace_ImgArrays *imgArraysPtr)
{
  switch(WRAPPER(target)) 
    {
    case GLACE_GIMP_PixelRgn:
      gimp_pixel_rgn_init(&WRAPPER(dest_rgn_ptr),
			  WRAPPER(drawable_ptr),
			  WRAPPER(gimp_x0), WRAPPER(gimp_y0),
			  GLACE_IMG(cols), GLACE_IMG(rows),
			  TRUE, TRUE);
      
      GLACE_IMG(putImgRowPtr) = WRAPPER(gimpImgPtr);
      break;
    case GLACE_GIMP_PREVIEW:
        {
          const unsigned int bpp = wData->preview->bpp;

          const unsigned int rowstride = 
              sizeof(Glace_IOGray) *
              GLACE_IMG(cols) *
              bpp;
      
          if (GLACE_IMG(preview_buffer) == NULL)
            {
              GLACE_IMG(preview_buffer) = g_new0(char, rowstride*
                                         GLACE_IMG(rows));
            }
        }
      break;
    default:
      g_warning("Lameness in WPutImgStart");
  } /* esac target */
}

/**************************************************************/
/*
 * :FUNCTION: Glace_WPutImgRowStart
 */
void
Glace_WPutImgRowStart(Glace_WData wData, 
		      Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		      Glace_ImgArrays *imgArraysPtr, 
		      int row G_GNUC_UNUSED)
{
  switch(WRAPPER(target)) 
    {
    case GLACE_GIMP_PixelRgn:
      break;
    case GLACE_GIMP_PREVIEW:
      GLACE_IMG(putImgRowPtr)=(Glace_IOGray *)GLACE_IMG(tmpImgPtr);
      break;
    default:
      g_warning("Lameness in WPutImgRowStart");
    } /* esac target */
}

/**************************************************************/


 /*
  * :FUNCTION: Glace_WPutImgRowFinish
  */
void
Glace_WPutImgRowFinish ( Glace_WData wData,
	                 Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
			 Glace_ImgArrays * imgArraysPtr,
			 int row )
{
  switch(WRAPPER(target))
    {
    case GLACE_GIMP_PixelRgn:
      /* Advance row pointer by rowstride */
      GLACE_IMG(putImgRowPtr) += GLACE_IMG(cols) * WRAPPER(drawable_ptr)->bpp;
      break;
    case GLACE_GIMP_PREVIEW:
        {
          const unsigned int bpp = wData->preview->bpp;

          const unsigned int rowstride = 
              sizeof(Glace_IOGray) *
              GLACE_IMG(cols) *
              bpp;
          
          memcpy (GLACE_IMG(preview_buffer)+row*rowstride, 
                  GLACE_IMG(putImgRowPtr),
                  rowstride);
        }      
      break;
    default:
      g_warning("Lameness in WPutImgRowFinish");
    } /* esac target */
}

void
Glace_Redraw_Preview (Glace_WData wData,
		      Glace_ImgArrays * imgArraysPtr)

{
  const unsigned int bpp = wData->preview->bpp;
  
  const unsigned int rowstride = 
    sizeof(Glace_IOGray) *
    GLACE_IMG(cols) *
    bpp;

    gint x1, y1;
    gint preview_width, preview_height;
    gint y_iter;
    char * viewable_prev_buffer, * view_ptr;
    gint view_rowstride;
    GimpPreview * preview =
        GIMP_PREVIEW(WRAPPER(preview)->preview);

    if (GLACE_IMG(preview_buffer) == NULL)
      {
        return;
      }
    
    gimp_preview_get_position (preview, &x1, &y1);
    gimp_preview_get_size (preview, &preview_width, &preview_height);

    view_rowstride = preview_width * bpp;
    viewable_prev_buffer = g_new0(char, 
                                  view_rowstride * preview_height);
    
    view_ptr = viewable_prev_buffer;
    for(y_iter = 0 ; y_iter < preview_height ; 
        y_iter++, view_ptr += view_rowstride)
      {
        memcpy (view_ptr,
                GLACE_IMG(preview_buffer) +
                     (y_iter+y1)*rowstride + x1*bpp,
                view_rowstride
               );
      }

    gimp_preview_draw_buffer (preview,
                              viewable_prev_buffer,
                              view_rowstride);

    g_free (viewable_prev_buffer);
}
/**************************************************************/


/*
 * :FUNCTION: Glace_WPutImgFinish
 */
void
Glace_WPutImgFinish ( Glace_WData wData,
		      Glace_CfgInfo *cfgInfoPtr G_GNUC_UNUSED,
		      Glace_ImgArrays * imgArraysPtr )
{
  switch(WRAPPER(target)) 
    {
    case GLACE_GIMP_PixelRgn:
      gimp_pixel_rgn_set_rect(&WRAPPER(dest_rgn_ptr),
			      WRAPPER(gimpImgPtr),
			      WRAPPER(gimp_x0), WRAPPER(gimp_y0),
			      GLACE_IMG(cols), GLACE_IMG(rows));
      
      gimp_drawable_flush (WRAPPER(drawable_ptr));
      gimp_drawable_merge_shadow (WRAPPER(drawable_id), TRUE);
      gimp_drawable_update (WRAPPER(drawable_id), 
			    WRAPPER(gimp_x0), WRAPPER(gimp_y0),
			    GLACE_IMG(cols), GLACE_IMG(rows));
      gimp_drawable_detach (WRAPPER(drawable_ptr));
      gimp_displays_flush();      
      break;
    case GLACE_GIMP_PREVIEW:
        {
          Glace_Redraw_Preview (wData, imgArraysPtr);

          /* FIXME : Free preview_buffer when imgArraysPtr is freed.
           * */
        }
      break;
    default:
      g_warning("Lameness in WPutImgFinish");
    } /* esac target */
}

/**************************************************************/
