/* $Id: preview.c,v 1.5 2000/06/19 06:15:05 acapnotic Exp $ */

#include <string.h>

#include "types.h"
#include "gimp_ace.h"
#include "preview.h"


/* Returns a pointer to a new GlaceGimpPreview.
 * If non-NULL, the provided preview and progress bar widgets will be used.
 * If NULL, new ones will be created. */
GlaceGimpPreview*
ace_preview_new(GlaceGimpData* const wData)
{
  GlaceGimpPreview* const self = g_new0(GlaceGimpPreview, 1);
  const gboolean is_colour = gimp_drawable_is_rgb(wData->drawable_id);
  GtkWidget* preview;
  GtkProgressBar* pbar;

  preview = 
      gimp_drawable_preview_new (wData->drawable_ptr, NULL);


  pbar = GTK_PROGRESS_BAR(gtk_progress_bar_new());
  self->preview = preview;
  self->pbar = GTK_PROGRESS_BAR(pbar);
  self->bpp = is_colour ? 3 : 1;
  self->idle_handle = 0;
  self->state = PREVIEW_IDLE;

  return self;
}

void 
ace_preview_update(Glace_ClientData* const clientData) 
{
  GlaceGimpPreview* const preview = clientData->wData->preview;

  if(ace_preview_get_state(preview) == PREVIEW_WORKING)
    {
      ace_preview_cancel(preview);
    }

  clientData->wData->target=GLACE_GIMP_PREVIEW;
  clientData->imgArraysPtr->pixelBytePad=0;

  ace_preview_progress_update(preview, (float)PROG_INPUT_CHUNKS/
			      (clientData->cfgInfoPtr->numTerms+PROG_OCHUNKS));
  ace_preview_set_state(preview, PREVIEW_WORKING);
  gimp_ace_go(clientData);
}

void
ace_preview_update_finished(GlaceGimpPreview* const preview)
{
  ace_preview_set_state(preview, PREVIEW_IDLE);
  ace_preview_progress_update(preview, 0.0);
  preview->idle_handle = 0;
}

void
ace_preview_cancel(GlaceGimpPreview* const preview)
{
  if(ace_preview_get_state(preview) == PREVIEW_WORKING) {
    g_source_remove(preview->idle_handle);
    ace_preview_update_finished(preview);
  }
}

void
ace_preview_progress_update(const GlaceGimpPreview* const preview, 
			    const gfloat percentage)
{
  gtk_progress_bar_set_fraction(preview->pbar, percentage);
}

AcePreviewState
ace_preview_get_state(const GlaceGimpPreview* const preview)
{
  return preview->state;
}

void
ace_preview_set_state(GlaceGimpPreview* const preview, 
		      const AcePreviewState ps)
{
  preview->state=ps;
}


/* For emacs:
Local Variables:
mode:c
c-file-style:"gnu"
End:
*/
